#-----------------------------------------------------------------------
#
#  Script to define kapaa specific data and procedures. This script
#  should be executed at global scope.
#
#  Procedures
#    kapaaMatl    -- return list of possible main or alternate
#		       materials
#    kapaaTips    -- return list of possible main or alternate
#		       tip sizes
#    kapaaSlices  -- return list of possible slice-heights
#
#  Data
#    kapaa:envelope   -- final part size envelope dimensions
#    kapaa:buildLimit -- maximum size of build envelope
#    kapaa:travHeight -- traverse height
#    kapaa:packGutterX -- air gap gutter when packing
#    kapaa:packGutterY -- air gap gutter when packing
#
#-----------------------------------------------------------------------

    ######################################
    #
    #   kapaa-specific data. All data is in inches.
    #

    # restrict slice heights to those in lister only
varedit sliceHeight -allowarbitrary false;

    # displayed envelope - max size of final part after shrinkage
    # These values must be smaller than the buildLimit values
    # to accommodate material shrinkage, support growth and
    # base oversize.
global kapaa:envelope;
if { [info exists kapaa:envelope] == 0 } {
    defvar kapaa:envelope {
	-type	    point3
	-label	    "Envelope dimensions"
	-default    { 10.59 10.59 12.28 }
	-readonly   true
    };
}
    # physical limits of the modeler
global kapaa:buildLimit;
if { [info exists kapaa:buildLimit] == 0 } {
    defvar kapaa:buildLimit {
	-type	    point3
	-label	    "Envelope build limit"
	-default    { 10.59 10.59 12.50 }
	-readonly   true
    };
}
global kapaa:travHeight;
if { [info exists kapaa:travHeight] == 0 } {
    defvar kapaa:travHeight {
	-type	    length
	-label	    "Traverse height"
	-default    0.026
	-readonly   true
    };
}

global kapaa:packGutterX;
if { [info exists kapaa:packGutterX] == 0 } {
    defvar kapaa:packGutterX { 
    	-type		length 
    	-default	0.02 
    	-label		"Gutter X" 
    	-balloon	"Gutter in X dir between jobs when packing" 
    	-min		0.0 
    	-max        2.0 
    }; 
};
global kapaa:packGutterY;
if { [info exists kapaa:packGutterY] == 0 } {
    defvar kapaa:packGutterY { 
    	-type		length 
    	-default	0.02 
    	-label		"Gutter Y" 
    	-balloon	"Gutter in Y dir between jobs when packing" 
    	-min		0.0 
    	-max        2.0 
    }; 
};
global kapaa:timeEstFactor;
if { [info exists kapaa:timeEstFactor] == 0 } {
    defvar kapaa:timeEstFactor {
    	-type		real
	-default	1.00
	-label		"Time estimate factor"
	-balloon	"Time estimate multiplier"
	-min		0.0
	-max		100.0
    };
};
    # interface controls
set ::autoCenterToolpath true;		# auto-center in packing?
set ::allowVWC           true;		# allow variable width contour?


########################################################################
#
#   PRIVATE
#	Returns a list of possible main or alternate materials, or
#	the name of the default main or alternate material. There
#	are two main forms for calling this function:
#
#	    kapaaMatl -main
#	    kapaaMatl -alt $mainType
#
#	where "mainType" would be a variable indicating the current
#	choice for main material.
#
#	Optionally, the keyword "-default" can be appended on the end
#	of the argument list to make the function return the default
#	main or alternate material.
#
#   RETURNS
#	When returning a material list, the list returned consists
#	of pairs where the first value is the mnemonic name of the
#	material, and the second value is the user-displayable
#	description.
#
#	When returning the default material, just the mnemonic name
#	is returned.
#
########################################################################
proc	kapaaMatl { matl { arg2 "" } { arg3 "" } } {

					# main material list
    if { $matl == "-main" } {
						# default
	if { $arg2 == "-default" } {
	    return P430;
						# material list
	} else {	   		
		return { {ABS-M30  "ABS-M30"} \
                 {ABS-M30i "ABS-M30i"} \
				 {P430 "ABS P430"} \
                 {ABS-ESD7 "ABS-ESD7"} \
                 {NYL12  "Nylon 12"} \
                 {PC     "PC"} \
				 {PC-ISO "PC-ISO"} \
				 {PC-ABS "PC-ABS"} \
				 {PPSF   "PPSF"} \
                 {ULT9085 "Ultem 9085"} \
                 {ULTM-BLK "Ultem 9085 Black"} };		
	}
					} elseif { $matl == "-alt" } {
						# default
	if { $arg3 == "-default" } {
	    switch -- $arg2 {
                ABS-M30 { return SR30; }
				P430 { return SR30; }
                ABS-M30i { return SR30; }
                ABS-ESD7 { return SR30; }
                ABS 	{ return SR20; }
                ABSi	{ return SR20; }
                PC-ABS  { return SR20; }
                PC 	{ return PC_S; }
                PC-ISO  { return PC_S; }
                PC-Translucent  { return PC_S; }
                PPSF 	{ return PPSF_S; }
                ULT9085	{ return ULT_S; }
                ULTM-BLK { return ULT_S; }
                NYL12   { return SR-110; }
                default { return SR30; } }
	} else {
						# material list
	    switch -- $arg2 {
    		ABS-M30 { return { {SR20   "SR20 support"}
                                   {SR30   "SR30 support"} }; }
                ABS-M30i { return { {SR20   "SR20 support"}
                                   {SR30   "SR30 support"} }; }
                ABS-ESD7 { return { {SR30  "SR30 support"} }; }
                ABS     { return { {SR20   "SR20 support"} }; }
                ABSi    { return { {SR20   "SR20 support"} }; }
                PC-ABS  { return { {SR20   "SR20 support"} }; }
    		PC 	{ return { {PC_S   "PC support"}
                                   {SR-100 "SR-100 support"} }; }
                PC-ISO  { return { {PC_S   "PC support"} }; }
                PC-Translucent  { return { {PC_S   "PC support"} }; }
    		PPSF 	{ return { {PPSF_S "PPSF support"} }; }
    		ULT9085	{ return { {ULT_S  "Ultem support"} }; }
    		ULTM-BLK { return { {ULT_S  "Ultem support"} }; }
                NYL12   { return { {SR-110 "SR-110 support"} }; }
				P430   { return { {SR30 "SR-30 support"} }; }
                default { return { {SR30   "SR30 support"} }; }
	    }
	}
    }

    errormsg "kapaaMatl: Bad material.";
    return "";
}; # kapaaMatl


########################################################################
#
#   PRIVATE
#	Returns a list of the valid sizes for the tip of the main
#	or alternate material. Tip sizes are formatted "Txx" where
#	xx is the size of the tip in millimeters.
#
#   ARGUMENTS
#	tip		Either "-main" or "-alt".
#	mainMaterial	Mnemonic for the main material type.
#	altMaterial	Mnemonic for the alternate material type.
#	default		Optional argument. If "-default", returns the
#			name of the default tip size.
#
#   RETURNS
#	The list of tip sizes for the material, or the default tip
#	size.
#
########################################################################
proc	kapaaTips { tip mainMaterial altMaterial { default "" } } {
				    # main material tip
    global modelTip;

    if { $tip == "-main" } {
	if { $default == "-default" } {		# default tip
	    switch -- $mainMaterial {
    		ABS-M30	{ return T14; }
    		ABS-M30i { return T16; }
    		ABS-ESD7 { return T16; }
    		ABS 	{ return T16; }
    		ABSi	{ return T16; }
    		PC	{ return T16; }
    		PC-ISO 	{ return T16; }
    		PC-ABS 	{ return T16; }
    		PC-Translucent 	{ return T16; }
    		PPSF 	{ return T16; }
    		ULT9085	{ return T16; }
    		ULTM-BLK { return T16; }
            NYL12   { return T16; }		
            P430   { return T14; }				
    		default { return T16; }
	    }
	} else {				# tip list
	    switch -- $mainMaterial {
    		ABS-M30 	{
                    if { $altMaterial == "SR20" } {
                        return { T10 T12 T14 T16 T20 };
                    } elseif { $altMaterial == "SR30" } {
                        return { T10 T12 T14 T16 T20 };
                    } else {
                        return { T16 };
                    }
                }
    		ABS-M30i	{
                    if { $altMaterial == "SR20" } {
                        return { T10 T12 T16 T20 };
                    } elseif { $altMaterial == "SR30" } {
                        return { T10 T12 T16 T20 };
                    } else {
                        return { T16 };
                    }
                }
    		ABS-ESD7 { return { T12 T16 }; }
    		ABS 	{
                    if { $altMaterial == "SR20" } {
                        return { T10 T12 T16 T20 };
                    } else {
                        return { T16 };
                    }
                }
    		ABSi	{
                    if { $altMaterial == "SR20" } {
                        return { T10 T12 T16 T20 };
                    } else {
                        return { T16 };
                    }
                }
    		PC      {
                    if { $altMaterial == "SR-100" } {
                        return { T10 T12 T16 };
                    } else {
                        return { T12 T16 T20 };
                    }
                }
    		PC-ISO 	{ return { T12 T16 T20 }; }
    		PC-ABS 	{ return { T10 T12 T16 T20 }; }
    		PC-Translucent 	{ return { T12 T16 }; }
    		PPSF 	{ return { T16 T20 }; }
    		ULT9085	{ return { T16 }; }
		    ULTM-BLK { return { T16 }; }  
            NYL12   { return { T12 T16 T20 }; }		
			P430   { return { T14 T16 }; }	
    		default { return { T16 }; }
	    }
	}
				    # alternate material tip
    } elseif { $tip == "-alt" } {
	  if { $default == "-default" } {		# default tip
	    switch -- $altMaterial {
    		SR20     { return T12; }
    		SR30     { return T16; }
    		SR-100   { return T12; }
			SR-110   { return T12; }
    		ABS_SS	 { return T12; }
    		PC_S	 { return T16; }
    		PC-ABS_S { return T12; }
    		PPSF_S	 { return T16; }
    		ULT_S    { return T16; }                
    		default  { return T16; }
	    }
	  } else {				# tip list
	    switch -- $altMaterial {
    		SR20	{ 
		    if { $modelTip == "T20" } {
		    	return { T12 };
		    } else {
		    	return { T12 };
		    }
                }
    		SR30	{ 
		    if { $modelTip == "T20" } {
		    	return { T12 T14 T16};
		    } else {
		    	return { T12 T14 T16};
		    }
                }
    		SR-100	{ 
		    if { $modelTip == "T20" } {
		    	return { T16 };
		    } else {
		    	return { T12 };
		    }
                }
    		ABS_SS	{ 
		    if { $modelTip == "T20" } {
		    	return { T12 };
		    } else {
		    	return { T12 };
		    }
                }
    		PC_S	{ 
		    if { $modelTip == "T20" } {
		    	return { T16 };
		    } elseif { $modelTip == "T16" } {
		    	return { T16 };
		    } else {
		    	return { T12 };
		    }
		}
    		PC-ABS_S {
		    if { $modelTip == "T20" } {
		    	return { T20 };
		    } else {
		    	return { T12 };
		    }
                }
    		PPSF_S	 { return T16; }
    		ULT_S    { return T16; }  
            SR-110 { return T12; }			
    		default  { return T16; }
	    }
	  }
    }

    errormsg "kapaaTips: Bad material.";
    return "";
}; # kapaaTips


########################################################################
#
#   PRIVATE
#	Returns the list of valid slice heights for the material/tip
#	combinations. The heights listed here are ones that are
#	tested, calibrated, and endorsed by Stratasys.
#
#   ARGUMENTS
#	mainMaterial	Mnemonic for the main material type.
#	mainTip		Size for the main material tip.
#	altMaterial	Mnemonic for the alternate material type.
#	altTip		Size for the alternate material tip.
#	request		Optional argument. If "-default", returns the
#			name of the default slice height. If "-limits",
#			returns a two-element list with the minimum and
#			maximum slice heights.
#
#   RETURNS
#	The list of valid slice heights, given the materials and tip
#	sizes, or the default slice height, or the slice height limits.
#
########################################################################
proc	kapaaSlices { mainMaterial mainTip altMaterial altTip {request ""}} {
    if { $request == "-limits" } {
        return { .004 .013 };
    } elseif { $request == "-default" } {
      if { $mainTip == "T16" } {
          return 0.013;
      } elseif { $mainTip == "T20" } {
          return 0.013;
      } elseif { $mainTip == "T12" } {
          return 0.007;
      } elseif { $mainTip == "T10" } {
          return 0.005;
	  } elseif { $mainTip == "T14" } {
          return 0.010 ;
      } else {
          return 0.010;
      }
    } else {            ## return list
      if { $mainTip == "T16" } {
          return { 0.010 0.013};
      } elseif { $mainTip == "T20" } {
          return { 0.013 };
      } elseif { $mainTip == "T12" } {
          return { 0.007 };
      } elseif { $mainTip == "T10" } {
          return { 0.005 };
	  } elseif { $mainTip == "T14" } {
          return { 0.005 0.007 0.010 0.013 };
      } else {
          return { 0.010 };
      }
    }
}; # kapaaSlices